@@ -10,7 +10,7 @@ use Class::AutoDB::Connect;
 use Class::AutoDB::Database;
 use Class::AutoDB::Registry;
 use Class::AutoDB::RegistryDiff;
-our $VERSION = '1.19_01';
+our $VERSION = '1.19_02';
 $VERSION=eval $VERSION;		# I think this is the accepted idiom..
 
 # NG 09-11-24: move Database first so AutoClass::get will not mask Database::get
@@ -233,7 +233,7 @@ Class::AutoDB - Almost automatic object persistence coexisting with human-engine
 
 =head1 VERSION
 
-Version 1.19_01
+Version 1.19_02
 
 =head1 SYNOPSIS
 
@@ -1412,9 +1412,8 @@ The 'drop' option to 'new' runs this method.
 When you declare a class to be persistent, we arrange for it to
 inherit from Class::AutoDB::Object by modifying its @ISA array. This
 has the effect of polluting your class's namespace with inherited
-methods.  It was a bad idea.  Sorry.  We plan to change this in a
-future release (currently planned for 1.30; see L<DEVELOPMENT
-ROADMAP>).
+methods.  It was a bad idea.  Sorry.  We hope to change this in a
+future release.
 
 Two of the inherited methods are commonly used. These can be invoked
 on a persistent object or a Class::AutoDB::Oid object. It's usually
@@ -1531,304 +1530,6 @@ when user classes employ multiple inheritance.
 
 =back
 
-=head1 DEVELOPMENT ROADMAP
-
-Subject to change, of course.  You will notice, I'm sure, that no dates are attached to releases.
-
-=begin html
-
-<h2>1.25 (items deferred from 1.20) - improve build &amp; test process to work better with CPAN</h2>
-<ul>
-<li>incorporate Test::Database</li>
-<li>add 'configure' to Build process to assist with choosing test database</li>
-</ul>
-<h2>1.30 multiple AutoDBs</h2>
-<ul>
-<li>Allow program to have multiple AutoDB objects, connected to different databases</li>
-<li>Objects fetched from a given database are, by default, put back there</li>
-<li>Probably also possible for ‘put’ to specify which database gets the object</li>
-<li>Another approach is to associate classes with specific AutoDBs</li>
-<ul>
-<li>this would handle our current hottest use case, which is to let Babel and Pipeline operate together but with different AutoDBs</li>
-</ul>
-<li>Big problem is schema registration. Presently, schemas are auto- registered in default database.  With new scheme, how does program conveniently control this?</li>
-</ul>
-<h2>1.40 namespace issues</h2>
-<ul>
-<li>clean up namespace imposed on user-objects (maybe in AutoClass, too)</li>
-<li>I tried in 1.20 to get partial solution by pushing Class::AutoDB::Object onto the end of user class's @ISA, instead of unshifting it onto the front. DOESN'T WORK!</li>
-<ul>
-<li>It does that okay, but introduces a new bug: oid generation and all that is doen by Serialize which is a base class of Object. </li>
-<li>With Object at the front of @ISA, that happens early; with Object at the end of @ISA, it happens late.  </li>
-<li>This screws up a lot of things.:(</li>
-</ul>
-<li>look at method resolution order (mro) material</li>
-<li>refactor Object, Serialize, and rethink where we generate oid and put object into cache</li>
-<ul>
-<li>Object shouldn't inherit from Serialize or AutoClass</li>
-<li>to cleanup namespace want Object to have minimal methods (perhaps just 'put' and 'oid' since these are needed for compatibility) and to come late in @ISA</li>
-<li>but, we need to generate oid and put object into cache early</li>
-<ul>
-<li>see discussion of bug introduced when I tried pushing Object onto end of @ISA.  point is that </li>
-</ul>
-<li>it seems clear we need to split these roles somehow..</li>
-</ul>
-<li>BTW  'get' applied to Oid accidentally does (almost) the right thing!</li>
-<ul>
-<li>fetches object, then runs 'get' method on thawed object</li>
-<li>'get', if not overridden, is AutoClass::get.  </li>
-<li>returns empty ARRAY or list, since no attributes specified</li>
-<li>'almost the obvious thing' - user probably expects object to be returned...</li>
-</ul>
-<li>it would be nice to continue providing convenience methods in user namespace: 'put', 'oid', and maybe 'autodb'.</li>
-<ul>
-<li>it must be okay for user to override these, which means our code must never call them. I think this is already the case</li>
-<li>the hard problem is what to do when invoked on Oids. </li>
-<li>presently, the Oid versions of these are hardcoded. to do it right, Oid code has to check whether user has overridden the method; if so, has to fetch and re-dispatch to user object</li>
-<li>OR: let user specify whether method, when invoked on Oid, has to fetch the object</li>
-<ul>
-<li>at present, 'put' on Oid does not fetch object; what if user wants to define 'put' method that should fetch object.  or wants to call 'put' something else but not fetch object</li>
-</ul>
-<li>instead of, or in addition to, 'autodb' method on user-objects, have AutoDB provide this as an exported function.</li>
-<ul>
-<li>think though implications when we allow multiple active AutoDB objects</li>
-</ul>
-<li>consider this scenario: we splice Object onto class A's @ISA; B inherits from A &amp; C; C overrides 'put'. will Perl see Object::put or C::put?</li>
-</ul>
-<li>small cleanups</li>
-<ul>
-<li>rationalize use of oid-&gt;{_CLASS}</li>
-<li>standardize idiom for allowing call as object or class method, or function</li>
-<ul>
-<li>standardize use of Serialize methods as class methods vs. functions</li>
-<li>perhaps include internal functions for case where call as function already known...</li>
-<li>accepted idiom is for wrapper to say 'goto &amp;inner'</li>
-</ul>
-<li>unify AutoDB &amp; Serialze interfaces as a first step toward making Serialize a standalone package</li>
-<ul>
-<li>give Serialize methods get, put, del, etc.</li>
-<li>maybe change Serialize fetch/store to get/put in the interest of consistency.</li>
-</ul>
-</ul>
-<li>arrange for Oids to delegate methods like 'can' and 'isa' to real class -- same scheme as for AutoHash</li>
-</ul>
-<h2> 1.50 general cleanup -- items will probably be moved around</h2>
-<ul>
-<li>'purge' - remove deleted objects from database - likely to slip to 1.30</li>
-<ul>
-<li>3 flavors of increasing cost</li>
-<ul>
-<li>purge from _AutoDB - trivial (but dangerous for app, since all future references to deleted object will generate the famous 'trying to deserialize...' error)</li>
-<ul>
-<li>DELETE FROM _AutoDB WHERE object IS NULL</li>
-</ul>
-<li>purge from collections</li>
-<ul>
-<li>use Registry to find all collections with keys of type 'object'</li>
-<li>update each collection to remove the offending objects</li>
-<li>for BaseTables, set column to NULL</li>
-<li>for ListTables, delete the row</li>
-</ul>
-<li>purge from objects</li>
-<ul>
-<li>get all objects into memory</li>
-<li>traverse each object looking for deleted objects</li>
-<li>not sure what to do with them: </li>
-</ul>
-</ul>
-</ul>
-<li>rethink decision to let 'put' leave oids to deleted objects in search keys</li>
-<ul>
-<li>rationale: obviously, cannot ensure that all oids in keys are for extant objects. why agonize over getting ti right for some keys?</li>
-<li>on the other hand, if user goes to the trouble of getting objects, deleting some elements, then putting objects back, perhaps we should be willing to clean-up the object on the way out the door...</li>
-</ul>
-<li>make Serialize a standalone package</li>
-<ul>
-<li>like AutoDB, maintains shared object structures</li>
-<li>doesn't have Collections</li>
-<li>this is first step toward supporting additional serialization options (eg, Storable), and different storage systems (eg, BerkeleyDB)</li>
-</ul>
-<li>add tests for connection management, eg, in 060.connections</li>
-<li>AutoDB methods to list objects (or maybe oids??) -- this is a 'maybe' item</li>
-<ul>
-<li>mem_objects -- all persistent objects in memory -- these are the ones that will be put by put_objects</li>
-<li>db_objects -- objects in database -- gets 'em all as Oids</li>
-<li>all_objects -- union of mem+db</li>
-<li>mem_only_objects (or maybe new_objects) -- objects in memory but not yet in db</li>
-<li>db_only_objects</li>
-<li>del_objects -- someday -- deleted objects -- here is where 'oids' makes more sense than 'objects'</li>
-</ul>
-<li>investigate class2colls bug - is this still open?</li>
-<ul>
-<li>Assumes registers happen top-down which is not guaranteed</li>
-<li>Top-down order will happen as expected if sub-classes use their parents.  I believe this is necessary anyway, else the attribute methods will not be defined…</li>
-</ul>
-<li>Table/MySQL types</li>
-<ul>
-<li>Refactor TYPES – currently duplicated in several modules – a real landmine waiting to go off!</li>
-<li>Provide utility methods for working with types</li>
-<li>Might be better to let Table do the database ops itself rather than just generating SQL – doesn’t matter much now, but may make statement handles easier to manage</li>
-<li>Allow arbitrary MySQL types</li>
-</ul>
-<li>AND’ing new query onto Cursor - low priority</li>
-<li>let alter drop columns - low priority</li>
-</ul>
-<h2>1.60 improved put/get; performance improvements</h2>
-<ul>
-<li>New thoughts on ‘put’ -- batch 'put' is super performance win</li>
-<ul>
-<li>Auto-put seems a hopeless goal.  Too bad…</li>
-<li>Improve efficiency of put_objects method and add ability to feed in list of objects. Should really be done as part of put-policy</li>
-<ul>
-<li>By default, $ob-&gt;put doesn’t write the object immediately, but rather puts it on a queue for later writing</li>
-<li>$autodb-&gt;put does actual writes. Called automatically by $autodb-&gt;close and maybe destructor if we can get this to work.</li>
-<li>In the full scheme of things, ‘put’ can specify queue so that objects can be written at different times. This could grow into a limited transaction capability.</li>
-<li>$obj-&gt;put could also specify whether current state of object should be written, or state as it exists when $autodb-&gt;put is called. The latter option is really another way of saying, “$obj is dirty” and is a backdoor way of approximating auto-put.</li>
-<li>For performance, $autodb-&gt;put presumably will sort database operations by table and do updates in batches</li>
-<li>Ideally want to group updates by table and do each table in bulk.</li>
-<li>This is going to be a pain – have to refactor code to collect SQL bits and pieces and put them together at the top. Will affect a lot of tests…</li>
-<li>Here’s a plausible design.</li>
-<ul>
-<li>Each table has methods to</li>
-<li>also need to collect list of tables with pending updates. This could be attached to the autodb</li>
-</ul>
-<li>Updates to _AutoDB.</li>
-<ul>
-<li>Class::AutoDB::Object:::put calls Class::AutoDB::Serialize::store on each object</li>
-</ul>
-<li>Updates to Collections</li>
-<ul>
-<li>Class::AutoDB::Object:::put calls Collection::put on each object which calls BaseTable::put and ListTable::put</li>
-</ul>
-</ul>
-<li>‘dirty bit’ put policy</li>
-<ul>
-<li>May be better to think of this as ‘deferred put’.</li>
-<li>When code decides that object is dirty, puts it on a list of objects to be ‘put’</li>
-<li>Later, perhaps during $autodb-&gt;put, the actual database updates occur</li>
-</ul>
-<li>Look at renaming put_objects to ‘put’. This will simply work.  Just need synonym and add to tests</li>
-<li>re-get (or maybe refresh) to get current value of object from database</li>
-<li>putting object with references to objects that are never put - at least mention this in POD</li>
-<ul>
-<li>not sure if this is a bug or a mis-feature</li>
-<li>in the SYNOPSIS example, suppose $joe is put, but his friends are not</li>
-<li>when 'Joe' is later retrieved, his friends will point to non-existent objects. derefing these gives rise to the dreaded "Trying to deserialize..." error</li>
-<li>the obvious solution is to automatically put such objects, but this leads to an asymmetry between the handling of new objects vs updates to old ones: the former are automatically 'put' while the latter are not...</li>
-<li>there's also the practical problem on implementing this feature without getting into an infinite loop, but I think I know how to avoid this</li>
-</ul>
-<li>Smarter putting (now obsolete)</li>
-<ul>
-<li>Decide on mechanism for detecting changed objects</li>
-<ul>
-<li>Explicitly set dirty bit – easiest for us, hardest for user</li>
-<li>Maintain copies of objects when read from database, and compare current value to copy. Have to do this for both the serialized and ‘relational-ized’ versions of object, since search keys can be virtual (e.g., ‘friends_names’ could reach into friends objects to get names).  Note: Data::Dumper has mode for storing keys in sorted order</li>
-<li>TIEd ARRAY and HASH implementations.  Probably not too hard, but users would have to use these classes instead of built-in types</li>
-</ul>
-<li>Implement read_only.  Or do this as part of put_policy</li>
-<li>put_policy—controls which objects are put</li>
-<ul>
-<li>auto vs. manual – auto probably means “during AutoDB::put_object”</li>
-<li>dirty vs. all vs. none (none would be set by read_only)</li>
-<li>reachable (yes/no) – yes means, “put all objects reachable from the start”</li>
-<li>Perhaps we should have system-wide default, plus ability to override in AutoDB, plus ability to override per-class, plus ability to override per-object</li>
-</ul>
-<li>Selectable persistence</li>
-<ul>
-<li>Make it possible to specify on a per object basis whether the object should be persistent. This would simplify the Registry code, for example, by allowing separate persistent vs. transient registries</li>
-</ul>
-</ul>
-</ul>
-<li>Performance improvements for 'get'</li>
-<ul>
-<li>may be useful to let object provide list of "related objects" and get them all at once</li>
-</ul>
-<li>Use statement handles in Serialize and Table to improve performance</li>
-<li>design concurrency/transaction scheme -- will be implemented later</li>
-</ul>
-<h2>1.70 better collections; class extents, etc.</h2>
-<ul>
-<li>class attributes </li>
-<ul>
-<li>not set when class 'used' in the course of fetching objects. see discussion under AutoClass</li>
-<li>AutoDB does not store class attributes</li>
-<li>related to simple way class attributes handled in AutoClass</li>
-</ul>
-<li>Test subclass w/o collection, i.e., transients only</li>
-<li>See if collection=&gt;__PACKAGE__ can work</li>
-<ul>
-<li>Have to construct table name from package name</li>
-<li>Convert :: to _ or something</li>
-<li>Worry about restriction on length</li>
-</ul>
-<li>Consider extending register to take 'classes' parameter (instead of just 'class'). looks easy, but how useful??</li>
-<li>Class extents</li>
-<ul>
-<li>implement as more or less standard list table, e.g. _AutoDB_classes(class,oid)</li>
-<li>Specify via a flag in %%AUTODB</li>
-<li>subclasses have to insert in parent extents</li>
-</ul>
-</ul>
-<h2>1.80 oid management</h2>
-<ul>
-<li>Current scheme -- generate unique id in Perl, use MySQL ‘replace’ to insert or update.  (Note: MySQL’s ‘insert… on duplicate key update’ is a better choice, but it’s only available from 4.1)  Problems:</li>
-<ul>
-<li>The database will be seriously corrupted if a new oid isn’t unique!!</li>
-<li>‘replace’ isn’t available in Postgres; will either need extra ‘select’ or stored procedure -- is this still true?</li>
-<li>'replace' is available in SQLite</li>
-</ul>
-<li>Alternative – get sequence number from database</li>
-<li>Another goal: ensure uniqueness of oids constructed on different machines, so databases can be safely merged...</li>
-<li>Option 1 – do it when object constructed</li>
-<ul>
-<li>Problem – extra trip to database</li>
-<li>Benefit – every update to object (even first) is a database ‘update’ – works for MySQL and Postgres</li>
-</ul>
-<li>Option 2 – do it when object stored</li>
-<ul>
-<li>Benefit – avoids extra trip to database</li>
-<li>Problem – self-referential objects are complicated.  If freeze code encounters the same object twice during serialization, there won’t be an oid yet.  The obvious recursion (if object doesn’t have oid, then store it) leads to infinite loop.  Probably need to maintain a hash of ‘in-progress’ objects; if freeze encounters an in-progress object, it has to do a dummy insert just to get the oid.</li>
-</ul>
-</ul>
-<h2>1.90 persistent big HASH and ARRAY</h2>
-<ul>
-<li>persistent HASH</li>
-<li>persistent ARRAY --  harder because of shifts &amp; splices!!</li>
-<ul>
-<li>(see also implementation of BerkeleyDB recno files)</li>
-<li>The obvious implementation is to give ARRAY elements an ‘array_index’ key, starting at 0.</li>
-<li>So long as you just do pushes and pops, the array will grow and shrink on high end, and the mapping from Perl array index to database array index is trivial</li>
-<li>But, when a shift is done, the element at the low end disappears and now the 0th Perl element has database index 1</li>
-<li>So, we need a translation mechanism that maps Perl index values into database index values</li>
-<li>In general, splice can introduce ‘holes’ and ‘bumps’ into the mapping: e.g., if you delete 10 elements from the middle of the ARRAY, the bordering Perl index values will differ by 10 in the database; or if you add 10 elements to the middle of the ARRAY, the database indexes assigned to these guys will be discontinuous from their neighbors</li>
-<li>Blech…</li>
-<li>Possible implementation: translation table is a list of &lt;count, database index&gt; pairs.  Meaning is that the first/next count elements have database indexes index, index+1,…, index+count-1</li>
-<li>In usual case (no general splices), table has one entry</li>
-</ul>
-<li>make sure design will work with concurrency</li>
-</ul>
-<h2>2.00 serialization engines</h2>
-<ul>
-<li>want to handle ties. at least, Hash::AutoHash and friends</li>
-<li>Storable may be viable</li>
-<ul>
-<li>looks like new 'attach' hook gives us what we need. another plus is that it handles ties.</li>
-<li>problem is that output is not human readable</li>
-</ul>
-<li>do a combination of Storable for the heavy lift and DD to produce readable form</li>
-<li>use Rmap to convert refs into Oids, producing a clean copy that can be directly DD'ed. will need the converse, too, of course.</li>
-<li>max_allowed_packet is a landmine waiting to go off</li>
-<ul>
-<li>possible solution is to split serialized forms into segments, each &lt; max size</li>
-<li>the 'replace' statement that updates _AutoDB will have to be smarter, too</li>
-</ul>
-</ul>
-<h2>2.10 port to SQLite or other DBMS</h2>
-<h2>3.00 concurrency/transactions</h2>
-
-
-=end html
-
 =head1 SUPPORT
 
 You can find documentation for this module with the perldoc command.
